<?

interface HTMLNode
{
    function getHTML();
};

class HTMLText implements HTMLNode
{
    private $_text;
    
    function HTMLText( $text )
    {
        $this->_text = $text;
    }

    function getHTML()
    {
        return $this->_text;
    }
};

abstract class HTMLElement implements HTMLNode
{
    private $_elements;
    private $_class;
    private $_id;
        
    function HTMLElement( $class = "", $id = "", $element = NULL )
    {
        $this->_elements = array();
        $this->_class = $class;
        $this->_id = $id;
        if( $element != NULL )
            $this->addElement( $element );        
    }
    
    function addElement( &$element )
    {        
        $this->_elements[] = $element;
    }
    
    function getHTML()
    {
        $result = "";
        if( count( $this->_elements ) == 0 )
            return $result;
            
        foreach( $this->_elements as $elem )
        {
            $result .= $elem->getHTML();            
        }
        return $result;
    }
    
    function getTagsHTML() 
    {
        $result = "";
        if( $this->_class != "" )
            $result .= "class=\"".$this->_class."\" ";
        if( $this->_id != "" )
            $result .= "id=\"".$this->_id."\"";
        return $result;
    }
    
    function getID()
    {
        return $this->_id;
    }
    
    function getClass() 
    {
        return $this->_class;
    }
};

class HTMLSpan extends HTMLElement 
{
    function HTMLSpan( $class = "", $id = "", $element = NULL )
    {
        parent::HTMLElement( $class, $id, $element );
    }
    
    function getHTML()
    {
        return "<span ".$this->getTagsHTML().">".parent::getHTML()."</span>";
    }
};

class HTMLLink extends HTMLElement
{
    private $_url;
    private $_title;
    private $_params;
    private $_target;
    
    function HTMLLink( $url, $title = "", $element = NULL )
    {
        parent::HTMLElement( "", "", $element );
        $this->_url = $url;
        $this->_title = $title;
        $this->_params = array();
        $this->_target = "";
    }
    
    function addParam( $key, $value )
    {
        $this->_params[$key] = $value;
    }
    
    function setTarget( $target )
    {
        $this->_target = $target;
    }
    
    function getHTML()
    {
        $paramlist = "";
        
        foreach( $this->_params as $key => $value )
        {
            $paramlist .= "$key=$value&";
        }
        
        return "<a href=\"".$this->_url.($paramlist == ""?"":"?$paramlist")."\" title=\"".$this->_title."\" target=".$this->_target.">".parent::getHTML()."</a>";
    }
};

class HTMLDiv extends HTMLElement
{
    function HTMLDiv( $class = "", $id = "", $element = NULL )
    {
        parent::HTMLElement( $class, $id, $element );
    }
    
    function getHTML()
    {
        return "<div ".$this->getTagsHTML().">".parent::getHTML()."</div>\n";
    }
};

class HTMLDivText extends HTMLDiv
{
    function HTMLDivText( $text, $class = "", $id = "", $element = NULL )
    {
        parent::HTMLDiv( $class, $id, $element );
        $this->addElement( new HTMLText( $text ) );
    }
};

class HTMLList extends HTMLElement
{
    function HTMLList( $class = "", $id = "", $element = NULL )
    {
        parent::HTMLElement( $class, $id, $element );
    }
    
    function getHTML()
    {
        return "<ul ".$this->getTagsHTML().">".parent::getHTML()."</ul>\n";
    }
};

class HTMLListEntry extends HTMLElement
{
    function HTMLListEntry( $class = "", $id = "", $element = NULL )
    {
        parent::HTMLElement( $class, $id, $element );
    }
    
    function getHTML()
    {
        return "<li ".$this->getTagsHTML().">".parent::getHTML()."</li>\n";
    }
};

class HTMLForm extends HTMLElement
{
    private $_action;
    private $_method;
    private $_upload;
    private $_submithandler;
    
    function HTMLForm( $action, $method, $upload = false, $class = "", $id = "", $element = NULL )
    {
        parent::HTMLElement( $class, $id, $element );
        $this->_action = $action;
        $this->_method = $method;        
        $this->_upload = $upload;
        $this->_submithandler = "";
    }
    
    function setSubmitHandler( $code )
    {
        $this->_submithandler = $code;
    }       
    
    function getSubmitHandlerCode()
    {
        if( $this->_submithandler != "" )
            return "onSubmit='".$this->_submithandler."'";
        else
            return "";
    }
    
    function getHTML()
    {
        $upload = $this->_upload ? "enctype=\"multipart/form-data\"": "";
        return "<form $upload action=\"".$this->_action."\" method=\"".$this->_method."\" ".
        $this->getSubmitHandlerCode().">\n".
            parent::getHTML()."</form>\n"; 
    }
};

class HTMLInput extends HTMLElement
{
    private $_name;
    private $_type;
    private $_size;
    private $_maxlength;
    private $_value;
    
    function HTMLInput( $name, $type, $size, $maxlength, $value, $class = "", $id = "" )
    {
        parent::HTMLElement( $class, $id, NULL );
        $this->_name = $name;
        $this->_type = $type;
        $this->_size = $size;
        $this->_maxlength = $maxlength;
        $this->_value = $value;
    }
    
    function getHTML()
    {
        return "<input name=\"".$this->_name."\" type=\"".($this->_type)
            ."\" size=\"".$this->_size."\" maxlength=\"".$this->_maxlength."\" value=\"".$this->_value."\" />\n";
    }
};

class HTMLHidden extends HTMLElement
{
    private $_name;
    private $_value;
    
    function HTMLHidden( $name, $value, $class = "", $id = "" )
    {
        parent::HTMLElement( $class, $id, NULL );
        $this->_name = $name;
        $this->_value = $value;
    }
    
    function getHTML()
    {
        return "<input type=\"hidden\" name=\"".$this->_name."\" value=\"".$this->_value."\" ".$this->getTagsHTML()." />\n";
    }
};

class HTMLButton extends HTMLElement
{
    private $_name;
    private $_submit;
    private $_caption;
    private $_clickhandler;
    
    function HTMLButton( $name, $submit, $caption, $class = "", $id = "" )
    {
        parent::HTMLElement( $class, $id, NULL );
        $this->_name = $name;
        $this->_submit = $submit;
        $this->_caption = $caption;
        $this->_clickhandler = "";
    }
    
    function setClickHandler( $code )
    {
        $this->_clickhandler = $code;
    }
    
    function getClickHandlerCode()
    {
        if( $this->_clickhandler != "" )
            return "onClick='".$this->_clickhandler."'";
        else
            return "";
    }
    
    function getHTML()
    {
        return "<input type=\"".($this->_submit?"submit":"button")."\" name=\"".$this->_name."\" value=\"".$this->_caption."\" ".$this->getClickHandlerCode()." />\n";
    }
};

class HTMLTextArea implements HTMLNode
{
    private $_name;
    private $_cols;
    private $_rows;
    private $_wrap;
    private $_content;
    
    function HTMLTextArea( $name, $cols, $rows, $wrap, $content, $class = "", $id = "" )
    {        
        $this->_name = $name;
        $this->_cols = $cols;
        $this->_rows = $rows;
        $this->_wrap = $wrap;
        $this->_content = $content;
    }
    
    function getHTML()
    {
        return "<textarea ".
            "name=\"".($this->_name)."\"".
            "cols=\"".($this->_cols)."\"".
            "rows=\"".($this->_rows)."\"".
            "wrap=\"".($this->_wrap)."\">".
            $this->_content.
            "</textarea>\n";
    }
};

class HTMLCheckBox extends HTMLElement
{
    private $_name;
    private $_value;
    private $_checked;
    
    function HTMLCheckBox( $name, $value, $checked = false, $class = "", $id = "" )
    {
        parent::HTMLElement( $class, $id, NULL );
        $this->_name = $name;
        $this->_value = $value;
        $this->_checked = $checked;
    }
    
    function getHTML()
    {
        return "<input type=\"checkbox\" name=\"".($this->_name)."\" value=\"".($this->_value)."\" ".($this->_checked?"checked":"")." />\n".
        parent::getHTML();
    }
};

class HTMLImage extends HTMLElement
{   
    private $_path;
    private $_width;
    private $_height; 
    private $_border;
    private $_align;
     
    function HTMLImage( $path, $width, $height, $border, $align="", $class = "", $id = "" )
    {
        parent::HTMLElement( $class, $id, NULL );
        $this->_path = $path;
        $this->_width = $width;
        $this->_height = $height;
        $this->_border = (string)$border;
        $this->_align = $align;
    }
    
    function getHTML()
    {
        $width = ($this->_width != "") ? "width=\"".$this->_width."\"" : "";
        $height = ($this->_height != "") ? "height=\"".$this->_height."\"" : "";
        $border = ($this->_border != "") ? "border=\"".$this->_border."\"" : "";
        $align = ($this->_align != "") ? "align=\"".$this->_align."\"" : "";
        
        return "<img src=\"".$this->_path."\" $width $height $border $align ".$this->getTagsHTML()." />";
    }
};

class HTMLCell extends HTMLElement
{
    function HTMLCell( $content, $class = "", $id = "" )
    {
        parent::HTMLElement( $class, $id, $content );
    }
    
    function getHTML()
    {
        return "<td valign=\"top\"".$this->getTagsHTML().">".parent::GetHTML()."</td>\n";
    }
}

class HTMLRow extends HTMLElement
{
    function HTMLRow( $cells, $class = "", $id = "" )
    {
        parent::HTMLElement( $class, $id, NULL );
        
        foreach( $cells as $cell )
        {
            $this->addElement( new HTMLCell( $cell ) );
        }        
    }
    
    function getHTML()
    {
        return "<tr ".$this->getTagsHTML().">\n".parent::GetHTML()."</tr>\n";
    }
};

class HTMLTable extends HTMLElement
{
    function HTMLTable( $rows, $class = "", $id = "" )
    {
        parent::HTMLElement( $class, $id, NULL );
        
        foreach( $rows as $row )
        {
            $this->addElement( new HTMLRow( $row ) );
        }        
    }
    
    function getHTML()
    {
        return "<table ".$this->getTagsHTML().">\n".parent::GetHTML()."</table>\n";
    }
};

class HTMLTemplate extends HTMLDiv
{
    function HTMLTemplate( $file, $map, $class = "", $id = "" )
    {
        $content = file_get_contents( "./".$file );
        
        foreach( $map as $key => $value )
        {
            $content = str_replace( $key, $value, $content );
        }
        
        parent::HTMLDiv( $class, $id, new HTMLText( $content ) );
    }
}

class HTML extends HTMLElement
{
    private $_doctype;
    private $_header;
    private $_title;
    private $_refs;
        
    function HTML( )
    {
        parent::HTMLElement();
        $this->_doctype = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
        
        $this->_header = array();
        $this->_refs = array();
        $this->_title = "";

        $this->addHeader( "Content-Type", "text/html; charset=iso-8859-1" );
        $this->addHeader( "Content-Language", "en-us" );
    }
    
    function addHeader( $equiv, $content )
    {
        $this->_header[] = "<meta http-equiv=\"$equiv\" content=\"$content\"/>";
    }
    
    function addStylesheet( $source, $media = "screen" )
    {
        $this->_refs[] = "<link rel=\"stylesheet\" href=\"$source\" media=\"$media\"/>";
    }
    
    function addTitle( $title )
    {
        $this->_title = "<title>$title</title>\n";
    }
  
    function createHeaderHtml( )
    {
        $result = $this->_title;
        foreach( $this->_header as $h )
        {
            $result .= "$h\n";
        }        
        foreach( $this->_refs as $r )
        {
            $result .= "$r\n";
        }        
        return $result;
    }
    
    function createBodyHtml() 
    {
        return parent::GetHTML();
    }
   
    function getPage()
    {
        return 
            $this->_doctype."\n".
            "<html xmlns=\"http://www.w3.org/1999/xhtml\" >\n".
            "<head>\n".
            $this->createHeaderHtml().
            "</head>\n".
            "<body>\n".
            $this->createBodyHtml().
            "</body>\n".
            "</html>\n";
    }
};
?>